@@ -26,7 +26,12 @@ __doc__ = "Installer for HPLIP tarball."
 
 
 # Std Lib
-import getopt, os, os.path, sys, time
+import getopt
+import os
+import os.path
+import sys
+import time
+import re
 
 # Local
 from base.g import *
@@ -42,6 +47,8 @@ USAGE = [(__doc__, "", "name", True),
          utils.USAGE_SPACE,
          utils.USAGE_OPTIONS,
          ("Automatic mode (chooses the most common options):", "-a or --auto", "option", False),
+         ("Force install of all dependencies (FOR TESTING ONLY):", "-x", "option", False),
+         ("Unknown distro mode (FOR TESTING ONLY):", "-d", "option", False),
          utils.USAGE_LOGGING1, utils.USAGE_LOGGING2, utils.USAGE_LOGGING3,
          utils.USAGE_HELP,
          utils.USAGE_SPACE,
@@ -61,14 +68,17 @@ log.set_module("hplip-install")
 log.debug("euid = %d" % os.geteuid())
 mode = INTERACTIVE_MODE
 auto = False
-test = False
+test_depends = False
+test_unknown = False
+language = None
 
 try:
-    opts, args = getopt.getopt(sys.argv[1:], 'hl:giawutx', 
-        ['help', 'help-rest', 'help-man', 'help-desc', 'gui',
+    opts, args = getopt.getopt(sys.argv[1:], 'hl:giawutxdq:', 
+        ['help', 'help-rest', 'help-man', 'help-desc', 'gui', 'lang=',
         'logging=', 'interactive', 'auto', 'web', 'browser', 'text']) 
 
-except getopt.GetoptError:
+except getopt.GetoptError, e:
+    log.error(e.msg)
     usage()
     sys.exit(1)
 
@@ -84,6 +94,9 @@ for o, a in opts:
 
     elif o == '--help-man':
         usage('man')
+        
+    elif o in ('-q', '--lang'):
+        language = a.lower()
 
     elif o == '--help-desc':
         print __doc__,
@@ -107,8 +120,12 @@ for o, a in opts:
         mode = BROWSER_MODE
         
     elif o == '-x':
-        test = True
+        log.warn("Install all depends (-x) is for TESTING ONLY")
+        test_depends = True
 
+    elif o == '-d':
+        log.warn("Unknown distro (-d) is for TESTING ONLY")
+        test_unknown = True
 
 log_file = os.path.normpath('./hplip-install_%s.log' % time.strftime("%a-%d-%b-%Y_%H:%M:%S"))
 
@@ -120,20 +137,36 @@ log.set_where(log.LOG_TO_CONSOLE_AND_FILE)
 
 log.debug("Log file=%s" % log_file)
 
+ac_init_pat = re.compile(r"""AC_INIT\(\[(.*?)\], *\[(.*?)\], *\[(.*?)\], *\[(.*?)\] *\)""", re.IGNORECASE)
+try:
+    config_in = open('./configure.in', 'r')
+except IOError:
+    prop.version = 'x.x.x'
+else:
+    for c in config_in:
+        if c.startswith("AC_INIT"):
+            match_obj = ac_init_pat.search(c)
+            prop.version = match_obj.group(2)
+            break
+
+    config_in.close()
+
 utils.log_title(__title__, __version__, True)
 
 log.info("Installer log saved in: %s" % log.bold(log_file))
 log.info("")
 
 if mode == BROWSER_MODE:
+    if test_depends or test_unknown:
+        log.error("Test modes -x and -d are not implemented with GUI mode.")
     from installer import web_install
     log.debug("Starting web browser installer...")
-    web_install.start()
+    web_install.start(language)
 
 elif mode == INTERACTIVE_MODE:
     from installer import text_install
     log.debug("Starting text installer...")
-    text_install.start(auto, test)
+    text_install.start(auto, test_depends, test_unknown)
 
 else:
     log.error("Invalid mode. Please use '-i', '-t', '-u' or '-w' to select the mode.")
