@@ -380,6 +380,10 @@ class LJJetReadyProxy;
 class LJFastRasterProxy;
 #endif
 
+#ifdef APDK_LJZJS_MONO
+class LJZjsMonoProxy;
+#endif
+
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
 class PSP100Proxy;
 class PSP470Proxy;
@@ -522,6 +526,10 @@ public:
     static LJFastRasterProxy s_LJFastRasterProxy;
 #endif
 
+#ifdef APDK_LJZJS_MONO
+    static LJZjsMonoProxy s_LJZjsMonoProxy;
+#endif
+
 #if defined(APDK_PSP100) && defined (APDK_DJ9xxVIP)
     static PSP100Proxy s_PSP100Proxy;
     static PSP470Proxy s_PSP470Proxy;
@@ -1118,6 +1126,21 @@ protected:
 	int			 FrPaperToMediaSize(PAPER_SIZE ps);
 }; //HeaderLJFastRaster
 
+//ClassName
+/*
+******************************************************************************/
+class HeaderLJZjs : public Header
+{
+    friend class LJZjs;
+public:
+    HeaderLJZjs (Printer *p, PrintContext *pc);
+    virtual DRIVER_ERROR Send ();
+    virtual DRIVER_ERROR FormFeed ();
+protected:
+    DRIVER_ERROR EndJob ();
+    DRIVER_ERROR SendCAPy (unsigned int iAbsY);
+}; // HeaderLJZjs
+
 class RasterSender : public Processor
 {
 friend class Header;
