@@ -22,7 +22,7 @@
 # Thanks to Henrique M. Holschuh <hmh@debian.org> for various security patches
 #
 
-__version__ = '9.0'
+__version__ = '11.0'
 __title__ = 'HP Device Manager'
 __doc__ = "The HP Device Manager (aka Toolbox) for HPLIP supported devices. Provides status, tools, and supplies levels."
 
@@ -36,27 +36,20 @@ import atexit
 
 # Local
 from base.g import *
-
 import base.utils as utils
+from base import service
 from base.msg import *
 
 log.set_module('hp-toolbox')
 
-# UI Forms and PyQt
-if not utils.checkPyQtImport():
-    log.error("PyQt/Qt initialization error. Please check install of PyQt/Qt and try again.")
-    sys.exit(1)
-
-from qt import *
-from ui.devmgr4 import DevMgr4
-
-
 app = None
 toolbox  = None
+loc = None
 
 USAGE = [(__doc__, "", "name", True),
          ("Usage: hp-toolbox [OPTIONS]", "", "summary", True),
          utils.USAGE_OPTIONS,
+         utils.USAGE_LANGUAGE,
          utils.USAGE_LOGGING1, utils.USAGE_LOGGING2, utils.USAGE_LOGGING3,
          utils.USAGE_HELP,
          utils.USAGE_SEEALSO,
@@ -81,9 +74,6 @@ def toolboxCleanup():
     pass
 
 def handleEXIT():
-    if hpiod_sock is not None:
-        hpiod_sock.close()
-
     if hpssd_sock is not None:
         hpssd_sock.close()
 
@@ -96,10 +86,11 @@ def handleEXIT():
 prop.prog = sys.argv[0]
 
 try:
-    opts, args = getopt.getopt(sys.argv[1:], 'l:hg', 
-        ['level=', 'help', 'help-rest', 'help-man', 'help-desc'])
+    opts, args = getopt.getopt(sys.argv[1:], 'l:hgq:', 
+        ['level=', 'help', 'help-rest', 'help-man', 'help-desc', 'lang='])
 
-except getopt.GetoptError:
+except getopt.GetoptError, e:
+    log.error(e.msg)
     usage()
 
 if os.getenv("HPLIP_DEBUG"):
@@ -128,35 +119,76 @@ for o, a in opts:
         print __doc__,
         sys.exit(0)
 
-
+    elif o in ('-q', '--lang'):
+        if a.strip() == '?':
+            utils.show_languages()
+            sys.exit(0)
+            
+        loc = utils.validate_language(a.lower())
 
 utils.log_title(__title__, __version__)
 
+# UI Forms and PyQt
+if not prop.gui_build:
+    log.error("GUI mode disabled in build. Exiting.")
+    sys.exit(1)
+    
+elif not os.getenv('DISPLAY'):
+    log.error("No display found. Exiting.")
+    sys.exit(1)
+
+elif not utils.checkPyQtImport():
+    log.error("PyQt init failed. Exiting.")
+    sys.exit(1)
+
+from qt import *
+from ui.devmgr4 import DevMgr4
+
 # Security: Do *not* create files that other users can muck around with
-os.umask (0077)
+os.umask (0037)
 
 # create the main application object
 app = QApplication(sys.argv)
 
-hpiod_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-try:
-    hpiod_sock.connect((prop.hpiod_host, prop.hpiod_port))
-except socket.error:
-    log.error("Unable to connect to HPLIP I/O (hpiod).")
-    sys.exit(1)
+if loc is None:
+    loc = user_cfg.ui.get("loc", "system")
+    if loc.lower() == 'system':
+        loc = str(QTextCodec.locale())
+        log.debug("Using system locale: %s" % loc)
 
-log.debug("Connected to hpiod on %s:%d" % (prop.hpiod_host, prop.hpiod_port))
+if loc.lower() != 'c':
+    log.debug("Trying to load .qm file for %s locale." % loc)
+    trans = QTranslator(None)
+    qm_file = 'hplip_%s.qm' % loc
+    log.debug("Name of .qm file: %s" % qm_file)
+    loaded = trans.load(qm_file, prop.localization_dir)
+    
+    if loaded:
+        app.installTranslator(trans)
+    else:
+        loc = 'c'
+
+        
+if loc == 'c':
+    log.debug("Using default 'C' locale")
+else:
+    log.debug("Using locale: %s" % loc)
+    QLocale.setDefault(QLocale(loc))
+    try:
+        locale.setlocale(locale.LC_ALL, locale.normalize(loc+".utf8"))
+        prop.locale = loc
+    except locale.Error:
+        log.error("Invalid locale: %s" % (loc+".utf8"))
 
-hpssd_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 try:
-    hpssd_sock.connect((prop.hpssd_host, prop.hpssd_port))
-except socket.error:
+    hpssd_sock = service.startup()
+except Error:
     log.error("Unable to connect to HPLIP I/O (hpssd).")
     sys.exit(1)
 
 log.debug("Connected to hpssd on %s:%d" % (prop.hpssd_host, prop.hpssd_port))
 
-toolbox = DevMgr4(hpiod_sock, hpssd_sock, toolboxCleanup, __version__)
+toolbox = DevMgr4(hpssd_sock, toolboxCleanup, __version__)
 app.setMainWidget(toolbox)
 
 toolbox.show()
@@ -164,14 +196,11 @@ toolbox.show()
 atexit.register(handleEXIT)
 signal.signal(signal.SIGPIPE, signal.SIG_IGN)
 
-user_config = os.path.expanduser('~/.hplip.conf')
-loc = utils.loadTranslators(app, user_config)
-
 try:
     log.debug("Starting GUI loop...")
     app.exec_loop()
 except KeyboardInterrupt:
-    pass
+    sys.exit(0)
 except:
     log.exception()
 
